==> filament-chan-dfu-0.0.10.tgz: package/download_release.js <==
// Load the AWS SDK for Node.js
var AWS = require('aws-sdk');
var exec = require("child_process").execSync;
/**
 * Don't hard-code your credentials!
 * Export the following environment variables instead:
 *
 * export AWS_ACCESS_KEY_ID='AKID'
 * export AWS_SECRET_ACCESS_KEY='SECRET'
 * export AWS_S3_REGION='us-west-2'
 */

// Set your region for future requests.
//AWS.config.region = 'us-west-2';
//process.env.AWS_PROFILE
AWS.config.region = "us-west-2";

var bucket="filament-firmware-beta";

var fs = require('fs');
var streamBuffers = require("stream-buffers")
var yauzl = require("yauzl");
var NO_DECRYPT = false;
var gcm ;
try {
  gcm = require('node-aes-gcm');
} catch(e){
  NO_DECRYPT = true;
}

const crypto = require('crypto');
const path = require('path');
const index_name = 'dfu_index.json';

function aes_kat() {
    key = new Buffer([0xfe, 0xff, 0xe9, 0x92, 0x86, 0x65, 0x73, 0x1c,
                      0x6d, 0x6a, 0x8f, 0x94, 0x67, 0x30, 0x83,
                      0x08]);
    plaintext = new Buffer([0xd9, 0x31, 0x32, 0x25, 0xf8, 0x84, 0x06,
                            0xe5, 0xa5, 0x59, 0x09, 0xc5, 0xaf, 0xf5,
                            0x26, 0x9a, 0x86, 0xa7, 0xa9, 0x53, 0x15,
                            0x34, 0xf7, 0xda, 0x2e, 0x4c, 0x30, 0x3d,
                            0x8a, 0x31, 0x8a, 0x72, 0x1c, 0x3c, 0x0c,
                            0x95, 0x95, 0x68, 0x09, 0x53, 0x2f, 0xcf,
                            0x0e, 0x24, 0x49, 0xa6, 0xb5, 0x25, 0xb1,
                            0x6a, 0xed, 0xf5, 0xaa, 0x0d, 0xe6, 0x57,
                            0xba, 0x63, 0x7b, 0x39, 0x1a, 0xaf, 0xd2,
                            0x55]);
    iv = new Buffer([0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce, 0xdb, 0xad,
                     0xde, 0xca, 0xf8, 0x88]);
    c = new Buffer([0x42, 0x83, 0x1e, 0xc2, 0x21, 0x77, 0x74, 0x24,
                    0x4b, 0x72, 0x21, 0xb7, 0x84, 0xd0, 0xd4, 0x9c,
                    0xe3, 0xaa, 0x21, 0x2f, 0x2c, 0x02, 0xa4, 0xe0,
                    0x35, 0xc1, 0x7e, 0x23, 0x29, 0xac, 0xa1, 0x2e,
                    0x21, 0xd5, 0x14, 0xb2, 0x54, 0x66, 0x93, 0x1c,
                    0x7d, 0x8f, 0x6a, 0x5a, 0xac, 0x84, 0xaa, 0x05,
                    0x1b, 0xa3, 0x0b, 0x39, 0x6a, 0x0a, 0xac, 0x97,
                    0x3d, 0x58, 0xe0, 0x91, 0x47, 0x3f, 0x59, 0x85]);
    t = new Buffer([0x4d, 0x5c, 0x2a, 0xf3, 0x27, 0xcd, 0x64, 0xa6,
                    0x2c, 0xf3, 0x5a, 0xbd, 0x2b, 0xa6, 0xfa, 0xb4]);

    e = gcm.encrypt(key, iv, plaintext, new Buffer([]));
    if (!e.ciphertext.equals(c)) {
        console.log ("Failed KAT");
        process.exit(1);
    }

    if (!e.auth_tag.equals(t)) {
        console.log ("Failed KAT");
        process.exit(1);
    }

    d = gcm.decrypt(key, iv, e.ciphertext, new Buffer([]),
                    e.auth_tag);
    if (!d.auth_ok) {
        console.log ("Failed KAT");
        process.exit(1);
    }

}

if (!NO_DECRYPT && gcm) aes_kat ();

function encrypt_dfu (file_path, key) {

    const iv = crypto.randomBytes(12);
    var file = fs.readFileSync(file_path);
    basename = path.posix.basename(file_path);
    var e = gcm.encrypt(key, iv, file, new Buffer(basename));
    e.ciphertext = e.ciphertext.toString('Base64');
    e.auth_tag = e.auth_tag.toString('Base64');
    e.iv = iv.toString('Base64');
    return e;
}


function get_index (callback) {
    var s3 = new AWS.S3({params: {Bucket: bucket, Key: index_name}});
    var i = s3.getObject(function(err, data) {
        if (err) {
            console.log(err, err.stack); // an error occurred
            process.exit(1);
        }
        else{
            index = JSON.parse(data.Body.toString());
            callback (index);
        }
    });
}

function put_index (index) {
    var s3obj = new AWS.S3({params: {Bucket: bucket, Key: index_name}});
    s3obj.upload({Body: index}).
        //on('httpUploadProgress', function(evt) { console.log(evt); }).
        send(function(err, data) {
            //console.log(err, data);
        });
}

function update_index (file_name) {
    get_index (function(index) {
        var entry = {};
        entry.name = file_name;
        entry.time = new Date().toISOString();
        index.history.push (entry);
        index.latest = file_name;
        put_index (JSON.stringify(index));
    });
}

function put_dfu(dfu_file_name) {
    const key = crypto.randomBytes(16);
    console.log ("This is the key for this release: %s", key.toString('base64'));
    const encrypted_dfu = encrypt_dfu (dfu_file_name, key);

    const s3name = path.posix.basename(dfu_file_name);
    var s3obj = new AWS.S3({params: {Bucket: bucket, Key: s3name}});
    s3obj.upload({Body: JSON.stringify(encrypted_dfu)}).
        //on('httpUploadProgress', function(evt) { console.log(evt); }).
        send(function(err, data) {
            //console.log(err, data);
            update_index (s3name);
        });
}

function get_dfu(b64key, opts, vers) {
  return new Promise((resolve, reject) => {
    function cb(err, res){
      if (err) return reject(err);
      console.log("resolving",res);
      resolve(res);
    }
    console.log("VERSION",vers);
    var getVersionFromIndex = (latest) => latest.split("-")[1].split(".z")[0] + "-release" // tap-v0.12.3.zip --> c0.12.3-release

    var getlocal = (relative) => {
      var fullpath = path.join(process.cwd(), relative);
      var pathparts = fullpath.split(path.sep);
      var version;
      while (pathparts.length){
        var section = pathparts.pop();
        if (section.indexOf('tap-v') === 0){
          version = section.split("-v")[1].split(" ")[0] + "-release"
          break;
        }
      }
      if (!version)
      {
        var dirpath = fullpath.split(path.sep);
        dirpath.pop();
        var cwd = dirpath.join(path.sep);
        var v1 = exec("git describe --abbrev=4 --dirty --always --tags", {cwd : cwd}).toString().trim();
        var v2 =  exec("git symbolic-ref --short -q HEAD", {cwd : cwd}).toString().trim()
        version =  v1 + "-" + v2
      }
      return ({
        binary : fs.readFileSync(fullpath),
        version : version
      })
    }

    console.log("VERSION",vers)

    if (!vers){
      var entries = {
        patchos : opts.patchos ? getlocal(opts.patchos) : null,
        tapos : opts.tapos ? getlocal(opts.tapos) : null,
        foible_application : opts.foible_application ? getlocal(opts.foible_application) : null
      }

      //all local, no s3
      if (Object.keys(entries).every(j => entries[j] ? true : false))
        return cb(null, entries);
    } else {
      var entries = {
        patchos : null,
        tapos : null,
        foible_application : null
      }
    }

    if (NO_DECRYPT) return cb(new Error("unable to load node-aes-gcm"))

    get_index(function(index){
        var s3 = new AWS.S3();
        var params = {Bucket: bucket, Key: vers || index.latest};

        var blob = new Buffer(0);
        console.log("getObject",index, vers);
        s3.getObject(params).
            on('httpData', function(chunk) {
                blob = Buffer.concat([blob, chunk]);
            }).
            on('httpDone', function() {
              console.log("httpdone")
              try {
                var encrypted_dfu = JSON.parse(blob.toString());
                console.log("parsed", b64key)
                  var key = new Buffer(b64key,'base64');
                   const iv = new Buffer(encrypted_dfu.iv, 'base64');
                   const tag = new Buffer(encrypted_dfu.auth_tag, 'base64');
                   const c = new Buffer(encrypted_dfu.ciphertext, 'base64');
                   console.log("pre decrypt");
                   var dfu = gcm.decrypt(key, iv, c, new Buffer(index.latest), tag);
                   console.log("post decrypt");
                  if (dfu.auth_ok == false) {
                      return cb(new Error("Invalid auth tag"));
                  }
                  function getsb (buf){
                    var StreamBuffer = new streamBuffers.ReadableStreamBuffer({
                      frequency: 10,   // in milliseconds.
                      chunkSize: 2048  // in bytes.
                    });

                     StreamBuffer.put(buf);
                     return StreamBuffer;
                  }
                  var StreamBuffer = getsb(dfu.plaintext);
                  console.log("yauzl")
                  yauzl.fromBuffer(dfu.plaintext,function(err,res){
                    if (err) return cb(err);

                    res.on('entry',(entry) => {
                      console.log(entry);
                      var fileName = entry.fileName;
                      var type = entry.type; // 'Directory' or 'File'
                      var size = entry.size;
                      console.log("entry", fileName)
                      Object.keys(entries).forEach(k => {
                        if (fileName.indexOf(`${k}.bin`) >= 0 && !entries[k]){
                          res.openReadStream(entry, function(err, readStream) {
                            if (err) return cb(err);
                            // ensure parent directory exists
                            var writebuffer = new streamBuffers.WritableStreamBuffer({});
                            readStream.pipe(writebuffer);
                            readStream.on('finish',() => {
                              console.log("readStream finish")
                              entries[k] = {
                                binary : writebuffer.getContents(),
                                version : (vers || index.latest).split("-v")[1].split(" ")[0] + "-release"
                              };
                              console.log(entries);
                              if (Object.keys(entries).every(j => entries[j] ? true : false)){
                                console.log("IF PASSED");
                                cb(null, entries)
                              }
                            })
                          });
                        }
                      })
                    })
                  })
              } catch (e) {cb(e)}

            }).
            send();
    });
  })

}


module.exports = {
  get_index : get_index,
  get_dfu : get_dfu
};

==> filament-chan-dfu-0.0.12.tgz: package/download_release.js <==
// Load the AWS SDK for Node.js
var AWS = require('aws-sdk');
var exec = require("child_process").execSync;
var concat = require("concat-stream");
/**
 * Don't hard-code your credentials!
 * Export the following environment variables instead:
 *
 * export AWS_ACCESS_KEY_ID='AKID'
 * export AWS_SECRET_ACCESS_KEY='SECRET'
 * export AWS_S3_REGION='us-west-2'
 */

// Set your region for future requests.
//AWS.config.region = 'us-west-2';
//process.env.AWS_PROFILE
AWS.config.region = "us-west-2";

var bucket="filament-firmware-beta";

var fs = require('fs');
var streamBuffers = require("stream-buffers")
var yauzl = require("yauzl");
var NO_DECRYPT = false;
var gcm ;
try {
  gcm = require('node-aes-gcm');
} catch(e){
  NO_DECRYPT = true;
}

const crypto = require('crypto');
const path = require('path');
const index_name = 'dfu_index.json';

function aes_kat() {
    key = new Buffer([0xfe, 0xff, 0xe9, 0x92, 0x86, 0x65, 0x73, 0x1c,
                      0x6d, 0x6a, 0x8f, 0x94, 0x67, 0x30, 0x83,
                      0x08]);
    plaintext = new Buffer([0xd9, 0x31, 0x32, 0x25, 0xf8, 0x84, 0x06,
                            0xe5, 0xa5, 0x59, 0x09, 0xc5, 0xaf, 0xf5,
                            0x26, 0x9a, 0x86, 0xa7, 0xa9, 0x53, 0x15,
                            0x34, 0xf7, 0xda, 0x2e, 0x4c, 0x30, 0x3d,
                            0x8a, 0x31, 0x8a, 0x72, 0x1c, 0x3c, 0x0c,
                            0x95, 0x95, 0x68, 0x09, 0x53, 0x2f, 0xcf,
                            0x0e, 0x24, 0x49, 0xa6, 0xb5, 0x25, 0xb1,
                            0x6a, 0xed, 0xf5, 0xaa, 0x0d, 0xe6, 0x57,
                            0xba, 0x63, 0x7b, 0x39, 0x1a, 0xaf, 0xd2,
                            0x55]);
    iv = new Buffer([0xca, 0xfe, 0xba, 0xbe, 0xfa, 0xce, 0xdb, 0xad,
                     0xde, 0xca, 0xf8, 0x88]);
    c = new Buffer([0x42, 0x83, 0x1e, 0xc2, 0x21, 0x77, 0x74, 0x24,
                    0x4b, 0x72, 0x21, 0xb7, 0x84, 0xd0, 0xd4, 0x9c,
                    0xe3, 0xaa, 0x21, 0x2f, 0x2c, 0x02, 0xa4, 0xe0,
                    0x35, 0xc1, 0x7e, 0x23, 0x29, 0xac, 0xa1, 0x2e,
                    0x21, 0xd5, 0x14, 0xb2, 0x54, 0x66, 0x93, 0x1c,
                    0x7d, 0x8f, 0x6a, 0x5a, 0xac, 0x84, 0xaa, 0x05,
                    0x1b, 0xa3, 0x0b, 0x39, 0x6a, 0x0a, 0xac, 0x97,
                    0x3d, 0x58, 0xe0, 0x91, 0x47, 0x3f, 0x59, 0x85]);
    t = new Buffer([0x4d, 0x5c, 0x2a, 0xf3, 0x27, 0xcd, 0x64, 0xa6,
                    0x2c, 0xf3, 0x5a, 0xbd, 0x2b, 0xa6, 0xfa, 0xb4]);

    e = gcm.encrypt(key, iv, plaintext, new Buffer([]));
    if (!e.ciphertext.equals(c)) {
        console.log ("Failed KAT");
        process.exit(1);
    }

    if (!e.auth_tag.equals(t)) {
        console.log ("Failed KAT");
        process.exit(1);
    }

    d = gcm.decrypt(key, iv, e.ciphertext, new Buffer([]),
                    e.auth_tag);
    if (!d.auth_ok) {
        console.log ("Failed KAT");
        process.exit(1);
    }

}

if (!NO_DECRYPT && gcm) aes_kat ();

function encrypt_dfu (file_path, key) {

    const iv = crypto.randomBytes(12);
    var file = fs.readFileSync(file_path);
    basename = path.posix.basename(file_path);
    var e = gcm.encrypt(key, iv, file, new Buffer(basename));
    e.ciphertext = e.ciphertext.toString('Base64');
    e.auth_tag = e.auth_tag.toString('Base64');
    e.iv = iv.toString('Base64');
    return e;
}


function get_index (callback) {
    var s3 = new AWS.S3({params: {Bucket: bucket, Key: index_name}});
    var i = s3.getObject(function(err, data) {
        if (err) {
            console.log(err, err.stack); // an error occurred
            process.exit(1);
        }
        else{
            index = JSON.parse(data.Body.toString());
            callback (index);
        }
    });
}

function put_index (index) {
    var s3obj = new AWS.S3({params: {Bucket: bucket, Key: index_name}});
    s3obj.upload({Body: index}).
        //on('httpUploadProgress', function(evt) { console.log(evt); }).
        send(function(err, data) {
            //console.log(err, data);
        });
}

function update_index (file_name) {
    get_index (function(index) {
        var entry = {};
        entry.name = file_name;
        entry.time = new Date().toISOString();
        index.history.push (entry);
        index.latest = file_name;
        put_index (JSON.stringify(index));
    });
}

function put_dfu(dfu_file_name) {
    const key = crypto.randomBytes(16);
    console.log ("This is the key for this release: %s", key.toString('base64'));
    const encrypted_dfu = encrypt_dfu (dfu_file_name, key);

    const s3name = path.posix.basename(dfu_file_name);
    var s3obj = new AWS.S3({params: {Bucket: bucket, Key: s3name}});
    s3obj.upload({Body: JSON.stringify(encrypted_dfu)}).
        //on('httpUploadProgress', function(evt) { console.log(evt); }).
        send(function(err, data) {
            //console.log(err, data);
            update_index (s3name);
        });
}

function get_dfu(b64key, opts, vers) {
  return new Promise((resolve, reject) => {
    function cb(err, res){
      if (err) return reject(err);
      console.log("resolving",res);
      resolve(res);
    }
    console.log("VERSION",vers);
    var getVersionFromIndex = (latest) => latest.split("-")[1].split(".z")[0] + "-release" // tap-v0.12.3.zip --> c0.12.3-release

    var getlocal = (relative) => {
      var fullpath = path.join(process.cwd(), relative);
      var pathparts = fullpath.split(path.sep);
      var version;
      while (pathparts.length){
        var section = pathparts.pop();
        if (section.indexOf('tap-v') === 0){
          version = section.split("-v")[1].split(" ")[0] + "-release"
          break;
        }
      }
      if (!version)
      {
        var dirpath = fullpath.split(path.sep);
        dirpath.pop();
        var cwd = dirpath.join(path.sep);
        var v1 = exec("git describe --abbrev=4 --dirty --always --tags", {cwd : cwd}).toString().trim();
        var v2 =  exec("git symbolic-ref --short -q HEAD", {cwd : cwd}).toString().trim()
        version =  v1 + "-" + v2
      }
      return ({
        binary : fs.readFileSync(fullpath),
        version : version
      })
    }

    console.log("VERSION",vers)

    if (!vers){
      var entries = {
        patchos : opts.patchos ? getlocal(opts.patchos) : null,
        tapos : opts.tapos ? getlocal(opts.tapos) : null,
        foible_application : opts.foible_application ? getlocal(opts.foible_application) : null
      }

      //all local, no s3
      if (Object.keys(entries).every(j => entries[j] ? true : false))
        return cb(null, entries);
    } else {
      var entries = {
        patchos : null,
        tapos : null,
        foible_application : null
      }
    }

    if (NO_DECRYPT) return cb(new Error("unable to load node-aes-gcm"))

    get_index(function(index){
      var s3 = new AWS.S3();
      var params = {Bucket: bucket, Key: vers || index.latest};
      console.log(index.latest, b64key);
      var blob = new Buffer(0);
      var decrypt = aesgcm.decrypt({key : b64key})
      s3.getObject(params)
        .createReadStream()
        .pipe(decrypt)
        .pipe(concat((plaintext) => {
          yauzl.fromBuffer(plaintext,function(err,res){
            if (err) return cb(err);

            res.on('entry',(entry) => {
              //console.log(entry);
              var fileName = entry.fileName;
              var type = entry.type; // 'Directory' or 'File'
              var size = entry.size;
              console.log("entry", fileName)
              Object.keys(entries).forEach(k => {
                if (fileName.indexOf(`${k}.bin`) >= 0 && !entries[k]){
                  res.openReadStream(entry, function(err, readStream) {
                    if (err) return cb(err);
                    // ensure parent directory exists
                    var writebuffer = new streamBuffers.WritableStreamBuffer({});
                    readStream.pipe(writebuffer);
                    readStream.on('finish',() => {
                      console.log("readStream finish")
                      entries[k] = {
                        binary : writebuffer.getContents(),
                        version : (vers || index.latest).split("-v")[1].split(" ")[0] + "-release"
                      };
                      if (Object.keys(entries).every(j => entries[j] ? true : false)){
                        cb(null, entries)
                      }
                    })
                  });
                }
              })
            })
          })
        }));
    });
	})
}


module.exports = {
  get_index : get_index,
  get_dfu : get_dfu
};
